from __future__ import annotations

from django.conf import settings
from django.core.mail import EmailMessage


class EmailHelper:
    def __init__(
        self,
        to: list | str,
        cc: list | None = None,
        bcc: list | None = None,
        fail_silently: bool = False
    ):
        if isinstance(to, str):
            self.to = [to]
        else:
            self.to = to

        self.cc = cc
        self.bcc = bcc
        self.from_email = settings.DEFAULT_FROM_EMAIL
        self.fail_silently = fail_silently


class SendGridEmailHelper(EmailHelper):
    def __init__(
        self,
        to: list | str,
        template_data: dict,
        template_id: str = settings.SENDGRID_TEMPLATE_ID,
        cc: list | None = None,
        bcc: list | None = None,
        fail_silently: bool = False
    ):
        super().__init__(to, cc, bcc, fail_silently)

        self.template_id = template_id
        self.template_data = template_data

    def send(self) -> None:
        msg = EmailMessage(
            from_email=self.from_email,
            to=self.to,
            cc=self.cc,
            bcc=self.bcc
        )
        msg.template_id = self.template_id
        msg.dynamic_template_data = self.template_data
        msg.send(fail_silently=self.fail_silently)



import shutil
import django_spire

from pathlib import Path
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError


INDENT = '    '


class AppManager:
    def __init__(self, base: Path, template: Path):
        self.base = base
        self.template = template

    def get_valid_root_apps(self) -> set[str]:
        return {
            app.split('.')[0]
            for app in settings.INSTALLED_APPS
            if '.' in app and app.split('.')[0] != 'django'
        }

    def is_valid_root_apps(self, components: list[str]):
        valid = self.get_valid_root_apps()
        root = components[0]

        if root not in valid:
            message = (
                f'Invalid root app "{root}". '
                f'The following are valid root apps: {", ".join(valid)}.'
            )

            raise CommandError(message)

    def validate_app_name_format(self, app: str):
        if '.' not in app:
            message = (
                'Invalid app name format. '
                'The app path must use dot notation (e.g., "parent.child").'
            )

            raise CommandError(message)

    def parse_app_name(self, app: str) -> list[str]:
        return app.split('.') if '.' in app else [app]

    def get_missing_components(
        self,
        components: list[str],
        registered_apps: list[str]
    ) -> list[str]:
        missing = []

        total = len(components)

        for i in range(total):
            app = '.'.join(components[: i + 1])

            if app not in registered_apps and i > 0:
                missing.append(app)

        return missing

    def create_custom_app(
        self,
        app: str,
        processor: 'TemplateProcessor',
        reporter: 'Reporter'
    ):
        components = app.split('.')
        destination = self.base.joinpath(*components)

        if destination.exists():
            reporter.report_app_exists(app, destination)
            return

        reporter.report_creating_app(app, destination)
        destination.mkdir(parents=True, exist_ok=True)

        if not self.template.exists():
            message = (
                f'Template directory "{self.template}" is missing. '
                'Ensure you have a valid app template.'
            )

            raise CommandError(message)

        shutil.copytree(
            self.template,
            destination,
            dirs_exist_ok=True,
            ignore=shutil.ignore_patterns('__pycache__', '*.pyc')
        )

        processor.replace_app_name(destination, components)
        reporter.report_app_creation_success(app)


class TemplateProcessor:
    def replace_app_name(self, directory: Path, components: str):
        app = components[-1]
        parent = components[-2] if len(components) > 1 else app
        module = '.'.join(components)

        for path in directory.rglob('*'):
            if path.is_file():
                self.replace_content(path, app, parent, module)
                self.rename_file(path, app, parent, module)

    def replace_content(self, path: Path, app: str, parent: str, module: str):
        replacements = self.generate_replacement(app, parent, module)

        with open(path, 'r', encoding='utf-8') as handle:
            content = handle.read()

        updated_content = self.apply_replacement(content, replacements)

        with open(path, 'w', encoding='utf-8') as file:
            file.write(updated_content)

    def rename_file(self, path: Path, app: str, parent: str, module: str):
        replacements = self.generate_replacement(app, parent, module)
        new_name = self.apply_replacement(path.name, replacements)

        if new_name != path.name:
            new_path = path.parent / new_name
            path.rename(new_path)

    @staticmethod
    def generate_replacement(
        app: str,
        parent: str,
        module: str
    ) -> dict[str, str]:
        return {
            'module': module,
            'Placeholder': app.capitalize(),
            'Placeholders': app.capitalize() + 's',
            'placeholder': app.lower(),
            'placeholders': app.lower() + 's',
            'Parent': parent.capitalize(),
            'Parents': parent.capitalize() + 's',
            'parent': parent.lower(),
            'parents': parent.lower() + 's'
        }

    @staticmethod
    def apply_replacement(text: str, replacements: dict[str, str]) -> str:
        for old, new in replacements.items():
            text = text.replace(old, new)

        return text


class Reporter:
    def __init__(self, command: BaseCommand):
        self.command = command

    def prompt_for_confirmation(self, message: str) -> bool:
        return input(message).strip().lower() == 'y'

    def report_missing_components(self, missing_components: list[str]):
        self.command.stdout.write(self.command.style.WARNING('The following are not registered apps:'))
        self.command.stdout.write('\n'.join(f' - {app}' for app in missing_components))

    def report_installed_apps_suggestion(self, missing_components: list[str]):
        self.command.stdout.write(self.command.style.NOTICE('\nPlease add the following to INSTALLED_APPS in settings.py:'))
        self.command.stdout.write('\n'.join(f'"{app}"' for app in missing_components))

    def report_app_creation_success(self, app: str):
        self.command.stdout.write(self.command.style.SUCCESS(f'Successfully created app: {app}'))

    def report_app_exists(self, app: str, destination: Path):
        self.command.stdout.write(self.command.style.WARNING(f'The app "{app}" already exists at {destination}'))

    def report_creating_app(self, app: str, destination: Path):
        self.command.stdout.write(self.command.style.NOTICE(f'Creating app "{app}" at {destination}'))

    def report_tree_structure(self, base: Path, components: list[str], registered_apps: list[str], template: Path):
        self.command.stdout.write('\nThe following app(s) will be created:\n\n')

        current = base

        for i, component in enumerate(components):
            current = current / component

            app = '.'.join(components[: i + 1])
            indent = INDENT * i

            self.command.stdout.write(f'{indent}📂 {component}/')

            if app not in registered_apps and template.exists():
                self.show_tree_from_template(template, indent=indent + INDENT)

    def show_tree_from_template(self, template_dir: Path, indent: str = ''):
        ignore = {'__init__.py', '__pycache__'}

        items = sorted(
            template_dir.iterdir(),
            key=lambda p: (p.is_file(), p.name.lower())
        )

        for item in items:
            if item.name in ignore:
                continue

            self.command.stdout.write(f'{indent}{"📁" if item.is_dir() else "📄"} {item.name}')

            if item.is_dir():
                self.show_tree_from_template(item, indent=indent + INDENT)


class Command(BaseCommand):
    help = 'Create a custom Spire app.'

    def __init__(self):
        super().__init__()

        self.base = Path(settings.BASE_DIR)
        self.template = Path(django_spire.__file__).parent / 'template/app'

        self.manager = AppManager(self.base, self.template)
        self.processor = TemplateProcessor()
        self.reporter = Reporter(self)

    def add_arguments(self, parser):
        parser.add_argument(
            'app_name',
            help='The name of the application to create',
            type=str
        )

    def get_app_names(self) -> list[str]:
        from django.apps import apps
        return [config.name for config in apps.get_app_configs()]

    def handle(self, *_args, **kwargs):
        app = kwargs.get('app_name')

        if not app:
            raise CommandError(self.style.ERROR('The app name is missing'))

        self.manager.validate_app_name_format(app)

        components = self.manager.parse_app_name(app)
        self.manager.is_valid_root_apps(components)

        registered_apps = self.get_app_names()

        self.stdout.write(self.style.NOTICE(f'Checking app components: {components}\n\n'))

        missing_components = self.manager.get_missing_components(components, registered_apps)

        if missing_components:
            self.reporter.report_missing_components(missing_components)
            self.reporter.report_tree_structure(self.base, components, registered_apps, self.template)

            if not self.reporter.prompt_for_confirmation('\nProceed with app creation? (y/n): '):
                self.stdout.write(self.style.ERROR('App creation aborted.'))
                return

            for module in missing_components:
                self.manager.create_custom_app(module, self.processor, self.reporter)

            self.reporter.report_installed_apps_suggestion(missing_components)
        else:
            self.stdout.write(self.style.SUCCESS('All component(s) exist!'))



from __future__ import annotations

from typing_extensions import TYPE_CHECKING

from django_spire.history.querysets import HistoryQuerySet

if TYPE_CHECKING:
    from django.db.models import QuerySet


class PlaceholderQuerySet(HistoryQuerySet):
    def active(self) -> QuerySet:
        return self.filter(is_active=True, is_deleted=False)



from __future__ import annotations

from django_spire.permission.models import PortalUser


def create_user(username: str, password: str) -> PortalUser:
    user = PortalUser.objects.create_user(username=username, password=password)
    user.save()

    return user


def create_super_user() -> PortalUser:
    user = PortalUser.objects.create_user(
        username='bobert@stratusadv.com',
        password='bobert'
    )

    user.is_superuser = True
    user.is_staff = True
    user.first_name = 'Bob'
    user.last_name = 'Robertson'
    user.email = 'bobert@stratusadv.com'

    user.save()

    return user



from __future__ import annotations

from django.contrib.auth.models import Group, User
from django.urls import reverse

from django_spire.breadcrumb.models import Breadcrumbs
from django_spire.history.mixins import ActivityLogMixin


class PortalGroup(Group, ActivityLogMixin):
    @classmethod
    def base_breadcrumb(cls) -> Breadcrumbs:
        crumbs = Breadcrumbs()
        crumbs.add_breadcrumb('Groups', reverse('permission:group_list'))
        return crumbs

    def breadcrumbs(self) -> Breadcrumbs:
        crumbs = Breadcrumbs()
        crumbs.add_base_breadcrumb(self._meta.model)

        if self.pk:
            crumbs.add_breadcrumb(
                name=self.name,
                href=reverse(
                    'permission:group_detail',
                    kwargs={'pk': self.pk}
                )
            )

        return crumbs

    class Meta:
        proxy = True
        verbose_name = 'Group'
        verbose_name_plural = 'Groups'


class PortalUser(User, ActivityLogMixin):
    @classmethod
    def base_breadcrumb(cls) -> Breadcrumbs:
        crumbs = Breadcrumbs()
        crumbs.add_breadcrumb('Users', reverse('user_account:list'))
        return crumbs

    def breadcrumbs(self) -> Breadcrumbs:
        crumbs = Breadcrumbs()
        crumbs.add_breadcrumb('User')

        if self.pk:
            crumbs.add_breadcrumb(
                name=self.get_full_name(),
                href=reverse('user_account:detail', kwargs={'pk': self.pk})
            )

        return crumbs

    class Meta:
        proxy = True
        verbose_name = 'User'
        verbose_name_plural = 'Users'



from __future__ import annotations

from typing_extensions import Callable, TYPE_CHECKING

from django import forms

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Submit

if TYPE_CHECKING:
    from django.contrib.auth.models import User


class ConfirmationForm(forms.Form):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.helper = FormHelper(self)
        self.helper.include_media = False
        self.helper.layout = Layout()
        self.helper.add_input(
            Submit(
                'submit',
                'Confirm',
                css_class='btn-success btn-sm mt-1 mb-0'
            )
        )


class DeleteConfirmationForm(ConfirmationForm):
    should_delete = forms.BooleanField(required=True)

    def __init__(self, *args, obj = None, **kwargs):
        if obj is None:
            message = 'Passing an object to DeleteConfirmationForm is required.'
            raise ValueError(message)

        self.obj = obj

        super().__init__(*args, **kwargs)

    def save(
        self,
        user: User,
        verbs: tuple,
        delete_func: Callable | None = None,
        activity_func: Callable | None = None,
        auto_add_activity: bool = True
    ) -> None:
        if delete_func is not None:
            delete_func()
        else:
            self.obj.set_deleted()

        if activity_func is not None:
            activity_func()
        elif hasattr(self.obj, 'add_activity') and auto_add_activity:
            self.obj.add_activity(
                user=user,
                verb=verbs[1],
                information=f'{user.get_full_name()} {verbs[1].lower()} {self.obj._meta.verbose_name} "{self.obj}".'
            )



from __future__ import annotations

from django.db import models
from django.utils.timezone import now

from django_spire.notification.enums import (
    NotificationSenderEnum,
    NotificationTypeChoices
)


class Notification(models.Model):
    type = models.CharField(max_length=32, default=NotificationTypeChoices.EMAIL)

    email = models.EmailField(default='')
    name = models.CharField(max_length=124, null=True, blank=True)
    title = models.CharField(max_length=128, default='')
    body = models.TextField(default='')
    url = models.CharField(max_length=255, default='')

    send_datetime = models.DateTimeField(default=now)

    created_datetime = models.DateTimeField(default=now)
    sent_datetime = models.DateTimeField(null=True, blank=True)
    is_sent = models.BooleanField(default=False)

    def mark_sent(self) -> None:
        self.is_sent = True
        self.sent_datetime = now()
        self.save()

    def send(self) -> None:
        sender_class = NotificationSenderEnum(self.type).value
        sender = sender_class(self)
        sender.send()

    class Meta:
        db_table = 'spire_notification'
        verbose_name = 'Notification'
        verbose_name_plural = 'Notifications'



from django.contrib import admin
from django.utils.html import format_html

from django_spire.notification import models


@admin.register(models.Notification)
class NotificationAdmin(admin.ModelAdmin):
    list_display = (
        'id', 'email', 'name', 'title', 'type', 'is_sent', 'send_datetime',
        'sent_datetime', 'view_body_snippet', 'url_link'
    )
    list_filter = ('type', 'is_sent', 'send_datetime')
    search_fields = ('id', 'email', 'name', 'title', 'type')
    ordering = ('-send_datetime',)

    def view_body_snippet(self, notification: models.Notification) -> str:
        return (
            notification.body[:20] + '...'
            if len(notification.body) > 20
            else notification.body
        )

    view_body_snippet.short_description = 'Body Snippet'

    def url_link(self, notification: models.Notification) -> str:
        if notification.url:
            return format_html(f'<a href="{notification.url}" target="_blank">Link</a>')

        return 'No URL'

    url_link.short_description = 'Notification URL'



from __future__ import annotations

from typing_extensions import Callable, TYPE_CHECKING

from django.http import HttpResponseRedirect
from django.template.response import TemplateResponse

from django_spire.breadcrumb.models import Breadcrumbs
from django_spire.form.confirmation_forms import DeleteConfirmationForm

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


# Takes breadcrumb as parameter and returns None
BreadCrumbCallable = Callable[[Breadcrumbs], None]


def detail_view(
    request: WSGIRequest,
    *,
    context_data: dict | None = None,
    breadcrumbs_func: BreadCrumbCallable | None = None,
    obj,
    template: str
) -> TemplateResponse:
    if context_data is None:
        context_data = {}

    breadcrumbs = Breadcrumbs()

    if breadcrumbs_func is None:
        breadcrumbs.add_obj_breadcrumbs(obj)
    else:
        breadcrumbs_func(breadcrumbs)

    base_context_data = {
        'page_title': obj._meta.model._meta.verbose_name,
        'page_description': obj.__str__(),
        'breadcrumbs': breadcrumbs
    }

    context_data = {**base_context_data, **context_data}

    return TemplateResponse(
        request,
        template=template,
        context=context_data
    )


def delete_form_view(
    request: WSGIRequest,
    *,
    context_data: dict | None = None,
    obj,
    activity_func: callable | None = None,
    auto_add_activity: bool = True,
    breadcrumbs_func: BreadCrumbCallable | None = None,
    delete_func: callable | None = None,
    # Present and past tense of verb
    verbs: tuple[str, str] = ('delete', 'deleted'),
    return_url: str,
    template: str = 'spire/page/form_full_page.html'
) -> HttpResponseRedirect | TemplateResponse:
    if context_data is None:
        context_data = {}

    model_name = obj._meta.model._meta.verbose_name

    if request.method == 'POST':
        form = DeleteConfirmationForm(request.POST)
        if form.is_valid():
            if delete_func is not None:
                delete_func()
            else:
                obj.set_deleted()

            if activity_func is not None:
                activity_func()
            elif hasattr(obj, 'add_activity') and auto_add_activity:
                obj.add_activity(
                    user=request.user,
                    verb=verbs[1],
                    device=request.device,
                    information=f'{request.user.get_full_name()} {verbs[1].lower()} {model_name} "{obj}".'
                )

            return HttpResponseRedirect(return_url)
    else:
        form = DeleteConfirmationForm()

    breadcrumbs = Breadcrumbs()

    if breadcrumbs_func is None:
        breadcrumbs.add_obj_breadcrumbs(obj)
        breadcrumbs.add_breadcrumb(name=verbs[0].title())
    else:
        breadcrumbs_func(breadcrumbs)

    base_context_data = {
        'page_title': obj.__str__(),
        'page_description': verbs[0].title(),
        'breadcrumbs': breadcrumbs,
        'form': form,
        'form_title': f'{verbs[0].title()} {obj}',
        'form_description': f'Are you sure you would like to {verbs[0].lower()} {model_name} "{obj}"?',
    }

    context_data = {**base_context_data, **context_data}

    return TemplateResponse(
        request,
        template=template,
        context=context_data
    )


def list_view(
    request: WSGIRequest,
    *,
    context_data: dict | None = None,
    breadcrumbs_func: BreadCrumbCallable | None = None,
    model,
    template: str
) -> TemplateResponse:

    if context_data is None:
        context_data = {}

    breadcrumbs = Breadcrumbs()

    if breadcrumbs_func is None:
        breadcrumbs.add_breadcrumb(name=f'{model._meta.verbose_name} List')
    else:
        breadcrumbs_func(breadcrumbs)

    base_context_data = {
        'page_title': model._meta.verbose_name,
        'page_description': 'List View',
        'breadcrumbs': breadcrumbs
    }

    context_data = {**base_context_data, **context_data}

    return TemplateResponse(
        request,
        template=template,
        context=context_data
    )


def form_view(
    request: WSGIRequest,
    *,
    form,
    context_data: dict | None = None,
    obj,
    breadcrumbs_func: BreadCrumbCallable | None = None,
    verb: str | None = None,
    template: str = 'spire/page/form_full_page.html'
) -> TemplateResponse:

    if context_data is None:
        context_data = {}

    model = obj._meta.model

    breadcrumbs = Breadcrumbs()

    if breadcrumbs_func is None:
        breadcrumbs.add_form_breadcrumbs(obj=obj)
    else:
        breadcrumbs_func(breadcrumbs)

    if verb is None:
        verb = 'Edit' if obj.pk else 'Create'

    if obj.pk:
        form_title = f'{verb.title()} {model._meta.verbose_name} {obj}'
        form_description = f'Are you sure you would like to {verb} {model._meta.verbose_name} "{obj}"?'
    else:
        form_title = f'{verb} {model._meta.verbose_name}'
        form_description = ''

    base_context_data = {
        'page_title': model._meta.verbose_name,
        'page_description': f'{verb} {obj}'.title(),
        'breadcrumbs': breadcrumbs,
        'form': form,
        'form_title': form_title,
        'form_description': form_description,
    }

    context_data = {**base_context_data, **context_data}

    return TemplateResponse(request, template, context=context_data)


def model_form_view(
    request: WSGIRequest,
    *,
    form,
    context_data: dict | None = None,
    obj,
    breadcrumbs_func: BreadCrumbCallable | None = None,
    verb: str | None = None,
    template: str = 'spire/page/form_full_page.html'
) -> TemplateResponse:
    if breadcrumbs_func is None:
        def breadcrumbs_func(crumbs: Breadcrumbs) -> None:
            if obj.pk is None:
                crumbs.add_form_breadcrumbs(obj)
            else:
                crumbs.add_form_breadcrumbs(obj)

    return form_view(
        request,
        form=form,
        context_data=context_data,
        obj=obj,
        breadcrumbs_func=breadcrumbs_func,
        verb=verb,
        template=template
    )


def template_view(
        request: WSGIRequest,
        page_title: str,
        page_description: str,
        breadcrumbs: Breadcrumbs,
        template: str,
        context_data: dict | None = None,
) -> TemplateResponse:
    if context_data is None:
        context_data = {}

    base_context_data = {
        'page_title': page_title,
        'page_description': page_description,
        'breadcrumbs': breadcrumbs
    }
    context_data = {**base_context_data, **context_data}

    return TemplateResponse(request, template, context=context_data)



from __future__ import annotations

from typing_extensions import ClassVar

from django import forms

from module import models


class PlaceholderForm(forms.ModelForm):
    field = forms.JSONField(required=False)

    class Meta:
        model = models.Placeholder
        fields: ClassVar = []



from __future__ import annotations

from typing_extensions import TYPE_CHECKING

from django.contrib.contenttypes.fields import GenericRelation
from django.db import models
from django.utils.timezone import localtime

from django_spire.history.models import (
    ActivityLog,
    ActivitySubscriber,
    EventHistory,
    View
)

if TYPE_CHECKING:
    from django.contrib.auth.models import User


class ActivityLogMixin(models.Model):
    activity_log = GenericRelation(
        ActivityLog,
        related_query_name='activity_log',
        editable=False
    )

    def add_activity(
            self,
            user: User,
            verb: str,
            information: str,
            recipient: User = None,
            subscribers: list[User] | None = None
    ) -> ActivityLog:

        activity = self.activity_log.create(
            user=user,
            verb=verb,
            information=information,
            recipient=recipient
        )

        if subscribers:
            subscriber_list = [
                ActivitySubscriber(user=subscriber, activity=activity)
                for subscriber in subscribers
            ]

            ActivitySubscriber.objects.bulk_create(subscriber_list)

        return activity

    class Meta:
        abstract = True


class HistoryModelMixin(ActivityLogMixin):
    is_active = models.BooleanField(default=True, editable=False)
    is_deleted = models.BooleanField(default=False, editable=False)

    activity_log = GenericRelation(
        ActivityLog,
        related_query_name='activity_log',
        editable=False
    )

    event_history = GenericRelation(
        EventHistory,
        related_query_name='event_history',
        editable=False
    )

    views = GenericRelation(
        View,
        related_query_name='view',
        editable=False
    )

    created_datetime = models.DateTimeField(default=localtime, editable=False)

    def add_view(self, user: User) -> None:
        self.views.create(user=user)

    def is_viewed(self, user: User) -> None:
        return self.views.filter(user=user).exists()

    @property
    def creator(self) -> User:
        return self.activity_log.earliest('date_time_entered').user

    def save(self, *args, **kwargs) -> None:
        created = self.pk is None
        super().save(*args, **kwargs)

        if created:
            self.event_history.create(event='crea')
        else:
            self.event_history.create(event='upda')

    def set_active(self) -> None:
        self.is_active = True
        self.save()
        self.event_history.create(event='acti')

    def set_deleted(self) -> None:
        self.is_deleted = True
        self.save()
        self.event_history.create(event='dele')

    def set_inactive(self) -> None:
        self.is_active = False
        self.save()
        self.event_history.create(event='inac')

    def un_set_deleted(self) -> None:
        self.is_deleted = False
        self.save()
        self.event_history.create(event='unde')

    class Meta:
        abstract = True



from __future__ import annotations

from django.contrib import messages
from django.http import HttpResponseRedirect
from django.urls import reverse

from django_spire.authentication.mfa.utils import get_or_generate_user_mfa_code


def mfa_notification_redirect_view(request):
    mfa_code = get_or_generate_user_mfa_code(request.user)

    if request.GET.get('resend', False):
        mfa_code.set_expired()
        mfa_code = get_or_generate_user_mfa_code(request.user)

    mfa_code.send_notification()
    messages.info(request, 'A authentication code has been sent to your email.')

    return HttpResponseRedirect(reverse('authentication:mfa:page:form'))



from django.urls import path

from django_spire.authentication.mfa.views import redirect_views


app_name = 'redirect'

urlpatterns = [
    path('notification/',
         redirect_views.mfa_notification_redirect_view,
         name='notification'),
]



# Generated by Django 5.1.3 on 2024-11-08 22:57

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.CharField(default='email', max_length=32)),
                ('email', models.EmailField(default='', max_length=254)),
                ('name', models.CharField(blank=True, max_length=124, null=True)),
                ('title', models.CharField(default='', max_length=128)),
                ('body', models.TextField(default='')),
                ('url', models.CharField(default='', max_length=255)),
                ('send_datetime', models.DateTimeField(default=django.utils.timezone.now)),
                ('created_datetime', models.DateTimeField(default=django.utils.timezone.now)),
                ('sent_datetime', models.DateTimeField(blank=True, null=True)),
                ('is_sent', models.BooleanField(default=False)),
            ],
            options={
                'verbose_name': 'Notification',
                'verbose_name_plural': 'Notifications',
                'db_table': 'spire_notification',
            },
        ),
    ]



from __future__ import annotations

from abc import ABC, abstractmethod
from typing_extensions import TYPE_CHECKING

from django_spire.notification.email import SendGridEmailHelper

if TYPE_CHECKING:
    from django_spire.notification.models import Notification


class NotificationSender(ABC):
    def __init__(self, notification: Notification):
        self.notification = notification

    @abstractmethod
    def send(self) -> None:
        raise NotImplementedError


class EmailNotificationSender(NotificationSender):
    def send(self) -> None:
        template_data = {
            'title': self.notification.title,
            'body': self.notification.body,
            'name': self.notification.name,
            'button_url': self.notification.url,
        }
        SendGridEmailHelper(
            to=self.notification.email,
            template_data=template_data,
            fail_silently=False
        ).send()



from django.urls import path

from module import views


app_name = 'placeholder'

urlpatterns = [
    path('<int:pk>/delete/form', views.placeholder_delete_view, name='delete'),
    path('<int:pk>/detail', views.placeholder_detail_view, name='detail'),
    path('<int:pk>/form/content', views.placeholder_form_content_view, name='form'),
    path('<int:pk>/form', views.placeholder_form_view, name='form'),
    path('list/', views.placeholder_list_view, name='list'),
]



# Generated by Django 5.1.3 on 2024-11-08 22:57

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='MfaCode',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(editable=False, max_length=6, unique=True)),
                ('expiration_datetime', models.DateTimeField(editable=False)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='mfa_codes', related_query_name='mfa_code', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'spire_authentication_mfa_code',
            },
        ),
    ]



from __future__ import annotations

import json

from django.contrib.contenttypes.models import ContentType
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from django.db.models import Model
    from django.http import HttpRequest
    from typing_extensions import Any


def get_object_or_null_obj(model: type[Model], pk: int, **kwargs) -> Model:
    try:
        return model.objects.get(pk=pk, **kwargs)
    except model.DoesNotExist:
        return model()


def get_object_or_none(model: type[Model], pk: int, **kwargs) -> Model:
    try:
        return model.objects.get(pk=pk, **kwargs)
    except model.DoesNotExist:
        return None


def process_request_body(request: HttpRequest) -> Any:
    body_unicode = request.body.decode('utf-8')
    return json.loads(body_unicode)['data']


def model_object_from_app_label(
    app_label: str,
    model_name: str,
    object_pk: int
) -> Model | None:
    try:
        content_type = ContentType.objects.get(
            app_label=app_label,
            model=model_name
        )
    except ContentType.DoesNotExist:
        return None

    model_class = content_type.model_class()
    return get_object_or_none(model_class, pk=object_pk)



from __future__ import annotations

import json

from django import forms
from django.contrib.auth.models import Group, User

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Row, Column, Submit

from django_spire.permission.factories import bulk_create_groups_from_names


class GroupNamesField(forms.CharField):
    """
        Receives a list of group names as a json string
    """
    def clean(self, value) -> list[str]:
        return json.loads(value)


class GroupNamesForm(forms.Form):
    groups = GroupNamesField(required=False)

    def save(self):
        return bulk_create_groups_from_names(self.cleaned_data['groups'])


class GroupForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.helper = FormHelper(self)
        self.helper.include_media = False
        self.helper.add_input(Submit('submit', 'Submit', css_class='btn-primary btn-sm bg-primary'))

    class Meta:
        model = Group
        exclude = ['permissions']
        widgets = {}


class GroupUserForm(forms.Form):
    def __init__(self, *args, **kwargs):
        group = kwargs.pop('group')
        super().__init__(*args, **kwargs)

        self.helper = FormHelper(self)
        self.helper.include_media = False
        self.fields['available_users'] = forms.ModelMultipleChoiceField(
            queryset=User.objects.exclude(id__in=[user.id for user in group.user_set.all()]),
            required=True,
        )
        self.fields['available_users'].label_from_instance = self.user_label
        self.helper.layout = Layout(
            Row(
                Column('available_users', css_class='form-group col-12'),
            ),
        )
        self.helper.add_input(Submit('submit', 'Submit', css_class='btn-primary btn-sm bg-primary'))

    @staticmethod
    def user_label(obj):
        return obj.get_full_name()



from __future__ import annotations

from django_spire.notification.models import Notification


def create_notification(
        email: str,
        title: str,
        body: str,
        url: str | None = None,
        name: str | None = None
) -> Notification:
    return Notification.objects.create(
        email=email,
        name=name,
        title=title,
        body=body,
        url=url
    )
